% HAM-ISI example - G1600726 controls lecture I
% Makes a HAM-ISI model along a horizontal degree of freedom (X or Y). The
% model is created in the state space format. 
% 
% This script makes 4 plots 
% 1) the transfer function bode plot
% 2) the impulse response
% 3) the fourier transform of the impulse response (to show it is the same as the bode plot) 
% 4) the poles in the complex plane
% 
% Brett Shapiro
% Last updated
% 28 March 2016

% clear all workspace variables, close all figures, and clear the command line
clear
close all
clc

%% Plot settings

% universal settings
textsize = 30;
linesize = 8;

% bode plot options
bodeopts = bodeoptions;
bodeopts.Grid = 'On';
bodeopts.FreqUnits = 'Hz';
bodeopts.MagUnits = 'abs';
bodeopts.MagScale = 'log';
bodeopts.Title.FontSize = textsize;
bodeopts.YLabel.FontSize = textsize;
bodeopts.XLabel.FontSize = textsize;
bodeopts.TickLabel.FontSize = 25;

%% HAM-ISI horizontal parameters, from G070156 slide 65
damp_ratio = 0.1; % damping ratio: 0 -> no damping, 1 -> critical damping; note the quality factor = 1/(2*damp_ratio).
g=9.81; % gravitational acceleration in m/s^2
m=1900; % mass of stage in kg
fl = 0.1482; % flexure length in m
Kxx = m*g/fl; % horizontal stiffness of stage
fx = sqrt(Kxx/m)/(2*pi); % Undamped horizontal natural frequency in Hz
Cxx = 2*damp_ratio*(2*pi*fx)*m; % viscous damping constant in N/(m/s)

%% State space model
A = [   0       1
     -Kxx/m  -Cxx/m];
B = [ 0
     1/m]; % force input
C = [1 0]; % displacement output
D = 0;
HAMmodel = ss(A,B,C,D); % state space model

% make the force to displacement bode plot of the transfer function; magnitude units are m/N
figure
h = bodeplot(HAMmodel,bodeopts);
title('HAM-ISI Example Bode Plot, force to displacement transfer function')

%% impulse response
Ts = 0.01; % time step, seconds
T_end = 100; % end time, seconds
time_vector = 0:Ts:T_end; % time vector, seconds

% calculate the impulse response
impulse_response_raw = impulse(HAMmodel,time_vector); % impulse response, arbitrary displacement units

% make the impulse response plot
figure,plot(time_vector,impulse_response_raw,'LineWidth',linesize),grid on
set(gca,'FontSize',textsize,'XLim',[0 10])
title('HAM-ISI model impulse response')
ylabel('Dispacement (arbitrary units)')
xlabel('Time (seconds)')

% calculate the FFT of the impulse response, to show it is the same as the transfer
% function bode plot
[resp,freqfft] = fft_response(impulse_response_raw,Ts);

% plot the FFT of the impulse response, to show it is the same as the transfer
figure
subplot(211)
    loglog(freqfft,abs(resp),'LineWidth',linesize),grid on
    set(gca,'FontSize',textsize,'YTick',10.^(-10:-4),'XLim',[0.1 10])
    ylabel('Magnitude')
    title('HAM-ISI model impulse response FFT')
subplot(212)
    semilogx(freqfft,angle(resp)*180/pi,'LineWidth',linesize),grid on
    set(gca,'FontSize',textsize,'YTick',-180:45:0,'XLim',[0.1 10])
    xlabel('Frequency (Hz)')
    ylabel('Phase (degrees)')

       
        
%% System poles
poles = eig(A); % poles of the HAM-ISI, units of rad/s
natural_frequency = abs(poles(1)); % rad/s, = 2*pi*fx
angle_from_imag_axis = angle(poles(1)) - pi/2; % angle of poles from the imaginary axis in the comlex plane, units of radians
damp_ratio_from_poles = sin(angle_from_imag_axis); % = damp_ratio variable above

% plot poles in complex plane
    % natural frequency circle
    angle_in_complex_plane = pi*(0:0.001:2);
    x_circle = natural_frequency*cos(angle_in_complex_plane);
    y_circle = natural_frequency*sin(angle_in_complex_plane);
    
    % line showing the angle of poles from imaginary axis
    x_angle = real(poles(1))*[0 1];
    y_angle = imag(poles(1))*[0 1];

    % make plot of poles in the complex plane
    figure
    plot(real(poles(1)),imag(poles(1)),'xb',... % pole 1
         real(poles(2)),imag(poles(2)),'xb',... % pole 2, which is the complex conjugate of pole 1
         10*[-1 1],[0 0],'--k',... % real axis line
         [0 0],10*[-1 1],'--k',... % imaginary axis line
         x_circle,y_circle,'--r',... % natural frequency circle
         x_angle,y_angle,'--m',... % line showing the angle of the poles
         'MarkerSize',40,'LineWidth',3)
     set(gca,'FontSize',textsize,'XTick',-10:2:10,'YTick',-10:2:10)
     xlabel('Real axis (rad/s)')
     ylabel('Imaginary axis (rad/s)')
     title('Complex plane pole map')
     axis square