% Sampling examples - controls lecture III
% This code illustrates why we need anti-alias and anti-image filters for
% the ADCs and DACs respectively.
% 
% 6 plots are generated:
% 1) An ADC sampled sine wave, with a higher frequeny (aliased) wave
% passing through the same sampling points.
% 2) An amplitude spectral density (ASD) of the sampled and original low and high
% frequency sine waves. The
% 3) The raw signal output by the DAC. The discrete steps occur because the
% DAC holds its voltage constant between samples.
% 4) The ASD of the raw DAC signal, showing all the high frequency
% harmonics generated by those discrete steps.
% 5) A bode plot of the anit-image filter we'll use to smooth the raw DAC
% signal.
% 6) The time domain plot of the DAC signal before and after the anti-image
% filter.
% 
% Brett Shapiro
% Last updated
% 12 April 2016

% clear all workspace variables, close all figures, and clear the command line
clear
close all
clc

%% Plot settings

% universal settings
textsize = 30;
linesize = 8;
markersize = 20;

% bode plot options
bodeopts = bodeoptions;
bodeopts.Grid = 'On';
bodeopts.FreqUnits = 'Hz';
bodeopts.MagUnits = 'abs';
bodeopts.MagScale = 'log';
bodeopts.Title.FontSize = textsize;
bodeopts.YLabel.FontSize = textsize;
bodeopts.XLabel.FontSize = textsize;
bodeopts.TickLabel.FontSize = 25;

%% Sampling parameters

time_step = 1/16; % seconds
    sampling_frequency = 1/time_step; % Hz
    Nyquist_frequency = sampling_frequency/2; % Hz, the highest frequency at which we can accurately measure sampled signals
time_end = 10; % seconds
    min_visible_frequency = 1/time_end; % Hz, this value isn't used here, just useful to know that the minimum frequency you can see is 1 / (length of the time window, in seconds)
sampled_time_vector = 0:time_step:time_end; % seconds

psuedo_continuous_time_step = 1/2^16; % 65536 Hz, must be much faster than time_step to simulate continuous time
psuedo_continuous_time_vector = 0:psuedo_continuous_time_step:time_end; % for plotting 'continuous time' comparisons of real signals to the sampled signals


%% ADC Sampled signals in the time domain

% low frequency signal
lowfreq_signal_frequency = 0.1*sampling_frequency; % Hz
sampled_lowfreq_signal = sin(2*pi*lowfreq_signal_frequency*sampled_time_vector);
original_lowfreq_signal = sin(2*pi*lowfreq_signal_frequency*psuedo_continuous_time_vector); % meant to be 'continuous' in time, which is approximated here with a must smaller time step then the sampling rate

% high frequency signal
highfreq_signal_frequency = lowfreq_signal_frequency + sampling_frequency; % Hz, adding the sample rate to the low frequency signal for the high frequency signal produces an interesting result
sampled_highfreq_signal = sin(2*pi*highfreq_signal_frequency*sampled_time_vector);
original_highfreq_signal = sin(2*pi*highfreq_signal_frequency*psuedo_continuous_time_vector); % meant to be 'continuous' in time, which is approximated here with a must smaller time step then the sampling rate

% sampled time history
figure
plot(psuedo_continuous_time_vector,original_lowfreq_signal,...
     sampled_time_vector,sampled_lowfreq_signal,'*',...
     psuedo_continuous_time_vector,original_highfreq_signal,...
     'LineWidth',linesize,'MarkerSize',markersize),grid on
set(gca,'FontSize',textsize,'XLim',[0 1/lowfreq_signal_frequency])
title([num2str(lowfreq_signal_frequency),' Hz signal sampled at ',num2str(sampling_frequency),' Hz'])
ylabel('Signal (volts)')
xlabel('Time (seconds)')
legend('Continuous time signal','Sampled signal','High frequency signal')


%% ADC Sampled signals in the frequency domain

% ASD of continuous low frequency signal
[ASD_original_lowfreq_signal,freq_continuous_time] = asd2(original_lowfreq_signal,psuedo_continuous_time_vector(2)-psuedo_continuous_time_vector(1),1,1,@hann);

% ASD of sampled low frequency signal
[ASD_sampled_lowfreq_signal,freq_sampled_signal] = asd2(sampled_lowfreq_signal,time_step,1,1,@hann);

% ASD of continuous high frequency signal
[ASD_original_highfreq_signal,~] = asd2(original_highfreq_signal,psuedo_continuous_time_vector(2)-psuedo_continuous_time_vector(1),1,1,@hann);

% ASD of sampled high frequency signal
[ASD_sampled_highfreq_signal,~] = asd2(sampled_highfreq_signal,time_step,1,1,@hann);

% plot amplitude spectral density
figure
loglog(freq_continuous_time,ASD_original_lowfreq_signal,...
       freq_sampled_signal,ASD_sampled_lowfreq_signal,...
       'LineWidth',linesize),grid on
hold on
loglog(freq_continuous_time,ASD_original_highfreq_signal,...
       freq_sampled_signal,ASD_sampled_highfreq_signal,'--',...
       Nyquist_frequency*[1 1],[1e-15 1],'--k',...
       'LineWidth',linesize),grid on
set(gca,'FontSize',textsize,'YLim',[1e-15 1],'XLim',[0.1 10000])
title(['Amplitude Spectral Density of ',num2str(lowfreq_signal_frequency),' Hz Signal with ',num2str(sampling_frequency),' Hz Sampling'])
ylabel('Amplitude (V/\surdHz)')
xlabel('Frequency (Hz)')
legend(['Continuous time ',num2str(lowfreq_signal_frequency),' Hz signal'],...
       ['Sampled ',num2str(lowfreq_signal_frequency),' Hz frequency signal'],...
       ['Continuous time ',num2str(highfreq_signal_frequency),' Hz signal'],...
       ['Sampled ',num2str(highfreq_signal_frequency),' Hz frequency signal'],...
       ['Nyquist Frequency = ',num2str(Nyquist_frequency),' Hz'])

   
%% Generating time domain DAC output with zero-oder-hold
  
time_step_ratio = time_step/psuedo_continuous_time_step; % ratio of sampling time step to 'continuous time' time step
    if round(time_step_ratio) ~= time_step_ratio % check that the two time steps overlap with each other
        warning('time_step_ratio is not an integer')
    end
    
DAC_output_signal = zeros(size(psuedo_continuous_time_vector)); % initialize output vector
sample_index = 1; % index 
DAC_output_signal(1) = sampled_lowfreq_signal(1);
for ii = 2:length(psuedo_continuous_time_vector)
    if mod(ii,time_step_ratio) == 0 && sample_index < length(sampled_time_vector)
        sample_index = sample_index + 1;
    end
    DAC_output_signal(ii) = sampled_lowfreq_signal(sample_index);
end

figure
plot(psuedo_continuous_time_vector,DAC_output_signal,...
     'LineWidth',linesize),grid on
set(gca,'FontSize',textsize,'YLim',1.01*[-1 1],'XLim',[0 2/lowfreq_signal_frequency])
ylabel('Signal (volts)')
xlabel('Time (seconds)')
title('Raw DAC signal')

 
%% DAC output frequency domain

% ASD of raw DAC output
[ASD_DACoutput,~] = asd2(DAC_output_signal,psuedo_continuous_time_step,1,1,@hann);

% plot amplitude spectral density of raw DAC output
figure
loglog(freq_continuous_time,ASD_DACoutput,...
       Nyquist_frequency*[1 1],[1e-15 1],'--k',...
       'LineWidth',linesize),grid on
set(gca,'FontSize',textsize,'YLim',[1e-10 2],'XLim',[0.1 10000])
title(['Amplitude Spectral Density of raw ',num2str(lowfreq_signal_frequency),' Hz DAC Output with ',num2str(sampling_frequency),' Hz Sampling'])
ylabel('DAC Output (V/\surdHz)')
xlabel('Frequency (Hz)')


%% DAC AI filtering

[ze,pe,ke] = ellip(4,1,40,2*pi*sampling_frequency*0.5,'s');
AIfilter = zpk(ze,pe,ke);
AIfilter = AIfilter/abs(freqresp(AIfilter,0)); % normalize the filter to have a gain of 1 at zero frequency

% Bode plot of AI filter
figure,bodeplot(AIfilter,bodeopts)

% filter data through AI filter using simulink
simlink_input = [transpose(psuedo_continuous_time_vector) transpose(DAC_output_signal)]; % input simulink data
sim('Simulink_AIfilter')
filtered_data = simlink_output.Data;

figure
plot(psuedo_continuous_time_vector,DAC_output_signal,...
     psuedo_continuous_time_vector,filtered_data,...
     'LineWidth',linesize,'MarkerSize',markersize),grid on
set(gca,'FontSize',textsize,'YLim',1.01*[-1 1],'XLim',[0 2/lowfreq_signal_frequency])
title(['DAC Output - Sample rate = ',num2str(sampling_frequency),' Hz'])
ylabel('Signal (volts)')
xlabel('Time (seconds)')
legend('Before Anti-Image','After Anti-Image')

